
/* (c) BSI 2004-2006. All Rights Reserved.

  This file is part of Dicop-Workerframe. For licencing information see the
  file LICENCE in the distribution, or http://www.bsi.bund.de/

*/
void hexdump(const unsigned char* title, const unsigned char* footer, const void *p, const unsigned int size, const unsigned char* indend);

